<?php
namespace MyAlbum;

  class Settings
  {
    /**
     * Holds the values to be used in the fields callbacks
     */
    private $options;
  
    /**
     * Start up
     */
    public function __construct()
    {
      add_action('admin_menu', array($this, 'addPage'));
      add_action('admin_init', array($this, 'pageInit'));
      add_action('admin_notices', array($this, 'tokenNotice'));
    }
        
    // Return setting by key
    public static function Get($key)
    {
      $options = get_option('myalbum');
      
      if(isset($options[$key]))
        return $options[$key];
      else
        return null;
    }
    
    // Show admin notice when token is missing or invalid
    public function tokenNotice()
    {
      if(current_user_can('manage_options'))
      {
        if(get_current_screen()->id==='settings_page_myalbum')
        {
          $request = Api::get('v1/member');
          set_transient('myalbum-tokenStatus', $request['status'], 0);
        }else
        {  
          $id = get_current_screen()->id;
          switch($id)
          {
            case 'dashboard':
            case 'plugins':
              $tokenStatus = get_transient('myalbum-tokenStatus');
              if((int)$tokenStatus===401 || $tokenStatus===false)
              {
                $class = 'notice notice-info';
                $link = admin_url('options-general.php?page=myalbum');
                $msg = sprintf(__("Almost there! Please <a href='%s'>submit a token</a> for your MyAlbum plugin.", "myalbum"), $link);
                
                printf('<div class="%1$s"><p>%2$s</p></div>', $class, $msg);
              }
            break;
          }
        }
      }
    }
    
  
    /**
     * Add options page
     */
    public function addPage()
    {
      // This page will be under "Settings"
      $options_page = add_options_page(
        __('MyAlbum settings', "myalbum"), 
        __('MyAlbum', "myalbum"), 
        'manage_options', 
        'myalbum', 
        array($this, 'createSettingsPage')
      );
      
      add_action('load-'.$options_page, array($this, 'pageLoaded'));
    }
    
    // Options page is loaded
    public function pageLoaded()
    {
      if(isset($_GET['hasToken']))
      {
        set_transient('myalbum-hasToken', 1, 0);
        wp_redirect(admin_url('options-general.php?page=myalbum'));
        exit();
      }
      
      $screen = get_current_screen();
      
      /*
      $screen->add_help_tab(array(
        'id'	=> 'overview',
        'title'	=> __('Overview'),
        'content'	=> '<p>' . __( 'Descriptive content that will show in My Help Tab-body goes here.', "myalbum") . '</p>'
      ));
      */
      
      get_current_screen()->set_help_sidebar(
      	'<p><strong>' . __('For more information:') . '</strong></p>' .
      	'<p>' . __('<a href="https://myalbum.com/wordpress">Documentation on the plugin</a>', "myalbum") . '</p>' .
      	'<p>' . __('<a href="https://myalbum.zendesk.com/hc/nl/">MyAlbum helpdesk</a>', "myalbum") . '</p>'
      );
    }
  
    /**
     * Options page callback
     */
    public function createSettingsPage()
    {
      // Set class property
      $this->options = get_option('myalbum');
      $p1 = __("An incredible immersive album experience embedded right into your Wordpress site. Install the free plugin and turn your MyAlbum account into a fully hosted photogallery. Transform photos, videos, and text into albums for free and embed them using the plugin.", "myalbum");
      $p2 = __("Free hosted photo gallery", "myalbum");
      $p3 = __("Use MyAlbum as your hosted photo gallery for free and create up to five albums.  Upgrade to Premium for unlimited albums. Albums are embedded as a \"summary view\" with a link to the full album on MyAbum. Upgrade to business to embed the complete album in your site without a link to MyAlbum.", "myalbum");
      $text = sprintf('%s<br/><br/><h2>%s</h2>%s', $p1, $p2, $p3);

      ?>
      <style>
        .myalbum .leader {
          background-color: black;
          width: auto;
          background: transparent url("<?php echo plugins_url("assets/images/banner-1544x500.jpg", PLUGINFILE); ?>") no-repeat center center;
          background-size: cover;
        }
        .myalbum .leader::before {
          content: '';
          display: inline-block;
          padding-bottom: 32.3%;
        }
        
        .myalbum H1 {
          margin-top: 20px;
        }
      </style>
      
      <div class="myalbum wrap">
        <div class="leader"></div>
        <h1><?php _e('MyAlbum', "myalbum");?></h1>
        <p><?php echo nl2br($text); ?></p><br/>
        <form method="post" action="options.php">
        <?php
            // This prints out all hidden setting fields
            settings_fields('myalbumSettings');
            
            // Prints out all settings sections added to a particular settings page.
            do_settings_sections('myalbum');
            submit_button(); 
        ?>
        </form>
      </div>
      <?php
    }
  
    /**
     * Register and add settings
     */
    public function pageInit()
    {    
      register_setting(
        'myalbumSettings',
        'myalbum',
        array($this, 'sanitize')
      );
      
      add_settings_section(
        'authentication',
        __('Settings', "myalbum"),
        array($this, 'printSectionInfo'),
        'myalbum' // Page
      );  

      add_settings_field(
        'token',
        __('Token', "myalbum"), 
        array($this, 'tokenCallback'), 
        'myalbum', 
        'authentication',
        array(
          'id' => 'token'
        )
      );
    }
  
    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     */
    public function sanitize($input)
    {
      $new_input = array();
      $errors = array();
      
      foreach($input as $key=>$value)
      {
        switch($key)
        {
          case 'token':
            $new_input['token'] = $input['token'];
            
            if(!$this->isTokenValid($input['token']))
              $errors[] = __('Unfortunately this token is incorrect.', "myalbum");
          break;
        }
      }
            
      if(count($errors)>0)
        add_settings_error("myalbumSettings", "myalbum-errors", implode("<br/>", $errors));
      
      return $new_input;
    }
    
    public function isTokenValid($token=null)
    {      
      $request = Api::get('v1/member', $token);      
      if($request['status']===200)
        return true;
    }
  
    /** 
     * Print the Section text
     */
    public function printSectionInfo($args)
    {
      switch($args['id'])
      {
        case "main":
          
        break;
      }
    }
    
    public function tokenCallback($args)
    {
      $value = isset($this->options[$args['id']]) ? htmlentities($this->options[$args['id']]) : '';
      $requestTokenUrl = ROOTURL.'/member/profile?redirect=1&createaccesstoken=1&description='.urlencode(get_bloginfo('url')).'&returnUrl='.urlencode(admin_url('options-general.php?page=myalbum&hasToken=1'));
      
      $autoFocus = false;
      $hasToken = false;
      if(get_transient('myalbum-hasToken')==1)
      {
        delete_transient('myalbum-hasToken');
        $value = '';
        $hasToken = true;
        $autoFocus = true;
      }
            
      if($hasToken)
      {
        
      }else if(!$this->isTokenValid($value))
      {
        // Invalid or missing token
        $icon = 'dashicons-warning';
        $color = '#dc3232';
        if(!empty($value))
        {
          // Invalid token
          $msg = __('Unfortunately this token is incorrect.', "myalbum").' '.__('We are unable to link your account without a valid token.', "myalbum");
        }else
        {
          // No token
          $msg = __("We are unable to link your account without a valid token.", "myalbum");
        }
      }else
      {
        // Valid token
        $color = '#46b450';
        $icon = 'dashicons-yes';
        $msg = __('Congratulations, your MyAlbum account has been linked. This plugin is now ready to use.', "myalbum");
      }
      
      if($autoFocus)
        $autoFocus = 'autofocus=true';
      
      $input = sprintf(
        '<input type="text" id="%1$s" name="myalbum[%1$s]" value="%2$s" %3$s>',
        $args['id'],
        esc_attr($value),
        $autoFocus
      );
      $msg = sprintf('<p style="font-size: 11px; color: %s;"><span class="dashicons %s" style="margin-top: 1px; font-size: 15px;"></span>%s<br/><br/></p>', $color, $icon, $msg);

      echo "<ol>";
      echo "<li><b>".sprintf(__("<a href='%s'>Request a token</a> from MyAlbum", "myalbum"), $requestTokenUrl)."</b><br/>".__("Use this token to safely link your MyAlbum account to this plugin.", "myalbum")."<div style='height: 15px;'></div></li>";
      echo "<li><b>".__("Enter your token", "myalbum")."</b><br/>{$input}{$msg}</li>";
      echo "</ol>";
      
    
      if(!empty($args['desc']))
        echo "<p class='description'>".$args['desc']."</p>";
    }
  }